#!/usr/bin/perl -w

# results.pl -- A CGI/Perl script.

#                 It returns a page with a list of executions and their status.
#                 If the execution is finished then it returns a web link for 
#                 their results.
#                 The results web page is automaticaly refreshed each N seconds.

# M.S.Reis, 25.03.2012

use strict;
use CGI;

use Data::Dumper;

# Base64
use MIME::Base64;

# Allows HTTP methods
require LWP::UserAgent;

# XML Parser
use XML::Simple;

#----- parameters -----

# Absolute path of this program's data directory
#
my $DATA_PATH = "/var/www/blast/data";

# www address of the download data
#
my $DATA_ADDRESS = "http://TestPageServer.com/blast/data";

# Input HTTP address
#
my $INPUT_ADDRESS = 'http://TestPageServer.com/cgi-bin/blast/results.pl';

# List of Execution IDs files; these files must exist, though they
# may be empty
#
my $EXEC_ID_FILE_IN = "exec_ids.in";
my $EXEC_ID_FILE_OUT = "exec_ids.out";

# Address of the execution status
#
my $STATUS_ADDRESS = 'http://InteGradeAPI.com:9090/execution';

# Time (in seconds) for the page refresh
#
my $N = 7;

#----- main program -----

# if the method is GET, it remove the contents of the exec_status files
# and ends the program
#
if ($ENV{REQUEST_METHOD} eq 'POST'){
 
  open (ARQ, ">$DATA_PATH/$EXEC_ID_FILE_IN") or die "Could not open $EXEC_ID_FILE_IN!\n";
  close (ARQ);
  open (ARQ, "$DATA_PATH/$EXEC_ID_FILE_OUT") or die "Could not open $EXEC_ID_FILE_OUT!\n";
  while (<ARQ>){
    if ($_ =~ /^(\d+)\s+\w+.*/){
      system ("rm -rf $DATA_PATH/$1");
    }
  }
  close (ARQ);
  open (ARQ, ">$DATA_PATH/$EXEC_ID_FILE_OUT") or die "Could not open $EXEC_ID_FILE_OUT!\n";
  close (ARQ);

  print "Content-type: text/html\n\n";
  print <<"HTML";
<HTML>
<HEAD>
<TITLE>Results frame</TITLE>
</HEAD>
<meta http-equiv="refresh" content="$N" />
<H3>Results</H3>
<form name="input" action="$INPUT_ADDRESS" method="post">
<p align="right"><input type="submit" value="Clean all results" /></p>
</form>
<hr>
HTML

  # end of CGI
  exit 0;
}


# if the method is GET, it prints all the execution status and results
# and ends the program
#

# reading the execution IDs data file
#
my %exec_status = (); # status of the execution (keys are the exec IDs)
my %exec_copies = (); # number of parametric copies (keys are the exec IDs)

open (ARQ, "$DATA_PATH/$EXEC_ID_FILE_IN") or die "Could not open $EXEC_ID_FILE_IN!\n";
while (<ARQ>){
  if ($_ =~ /^(\d+)\s+(\d+)/){
    $exec_status{$1} = "RUNNING";
    $exec_copies{$1} = $2;
  }
}	
close (ARQ);
open (ARQ, "$DATA_PATH/$EXEC_ID_FILE_OUT") or die "Could not open $EXEC_ID_FILE_OUT!\n";
while (<ARQ>){
  if ($_ =~ /^(\d+)\s+(\w+).*/){
    $exec_status{$1} = $2;
  }
}
close (ARQ);


# consulting the execution status for each execution ID that stills running
#
my $exec_id = -1;
my $userAgent = LWP::UserAgent->new;
$userAgent->timeout(30);
$userAgent->env_proxy;

my %files; # stores the output file names (the exec IDs are the hash keys)

foreach my $key (keys %exec_status){

  if ($exec_status{$key} eq "RUNNING"){

    my $response = $userAgent->get("$STATUS_ADDRESS/$key");
    $response->is_success or ($response->status_line =~ /^404/) or die $response->status_line;

    if ($response->as_string =~ /\<status\>FINISHED/){

      (-d "$DATA_PATH/$key") or mkdir ("$DATA_PATH/$key") 
                             or die "Could not create output directory for execution ID $key\n";
      system ("rm -Rf $DATA_PATH/$key/*");

      my $result_response = $userAgent->get("$STATUS_ADDRESS/$key/result");
      $result_response->is_success or die $result_response->status_line;
      my $xml_data = $result_response->as_string;
      $xml_data =~ s/^(?:.*\n){1,9}//;
      my $result_data = XMLin ($xml_data);
      my $i = 0;
 
      if (ref($result_data->{outputFiles}->{executionFile}) eq 'ARRAY'){

        # we set an execution as finished iff all the output files are finished 
        # 
        if (scalar(@{$result_data->{outputFiles}->{executionFile}}) == $exec_copies{$key}){
          $exec_status{$key} = "FINISHED";
        }

        foreach my $execution_file (@{$result_data->{outputFiles}->{executionFile}}){
           my $file_name = $DATA_PATH . "/" . $key . "/" . $execution_file->{fileName};
           open (ARQ, ">$file_name") or die "Could not create output file of execution ID $key\n";
           printf ARQ "%s", decode_base64 ($execution_file->{file});
           close (ARQ);
        }
      }
      else{ 
        if ($exec_copies{$key} == 1){
          $exec_status{$key} = "FINISHED";
        }
        my $file_name = $DATA_PATH . "/" . $key . "/" . $result_data->{outputFiles}->{executionFile}->{fileName};
        open (ARQ, ">$file_name") or die "Could not create output file of execution ID $key\n";
        printf ARQ "%s", decode_base64 ($result_data->{outputFiles}->{executionFile}->{file});
        close (ARQ);   
      }
    }
  }

  # if the execution is finished then we'll store the output file names 
  # and later use the hash to create the web links to them
  # 
  if ($exec_status{$key} eq "FINISHED"){ 
    opendir(DIR, "$DATA_PATH/$key") or die "Could not read files from the output directory!\n";
    while (my $file = readdir(DIR)){
      next if ($file =~ m/^\./);  # ignores files starting with .
      $files{$key}->{$file} = 1; 
    }
    closedir(DIR);
  }
}	


# writing the new execution IDs data file
#
open (ARQ, ">$DATA_PATH/$EXEC_ID_FILE_OUT") or die "Could not open $EXEC_ID_FILE_OUT!\n";
foreach my $key (sort keys %exec_status){
  printf ARQ "%d %s\n", $key, $exec_status{$key};
}	
close (ARQ);


# creating the HTML document 
#
print "Content-type: text/html\n\n";
print <<"HTML";
<HTML>
<HEAD>
<TITLE>Results frame</TITLE>
</HEAD>
<meta http-equiv="refresh" content="$N" />
<H3>Results</H3>
<form name="input" action="$INPUT_ADDRESS" method="post">
<p align="right"><input type="submit" value="Clean all results" /></p>
</form>
<hr>
HTML

foreach my $key (sort {$a <=> $b} keys %exec_status){
  if ($exec_status{$key} eq "FINISHED"){
    printf "Execution %d status: <font color='green'>%s</font> -&nbsp;\n", $key, $exec_status{$key};
  }
  elsif ($exec_status{$key} eq "RUNNING"){
    printf "Execution %d status: <font color='yellow'>%s</font>\n", $key, $exec_status{$key};
  }
  foreach my $file_name (sort keys %{$files{$key}}){
    printf "[<a href='$DATA_ADDRESS/$key/$file_name' target='_blank'>$file_name</a>]&nbsp;\n";
  }
  print "<br />\n";
}

# end of CGI
exit 0;

